#!/usr/bin/env python3
# *-* coding: utf-8 *-*

import subprocess, argparse, json

def create_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-p","--pkgs",
        metavar="package name",
        help="packages to remove (seperate with space)",
        nargs="+"
        )
    parser.add_argument(
        "-f","--file",
        metavar="Filename",
        help="Use json file"
        )
    parser.add_argument(
        "-s","--show-configs",
        help="Show available configs from json file",
        nargs="?",
        const=True
        )
    parser.add_argument(
        "-c","--config",
        help="Use this config from json file",
        metavar="config"
        )
    parser.add_argument(
        "--skip-device-check",
        help="Skip adb devices check. Only useful if your sure device is connected and found by adb",
        nargs="?",
        const=True
        )
    parser.add_argument(
        "--dump-json-file",
        help="Show json file",
        nargs="?",
        const=True
        )
    parser.add_argument(
        "--verbose",
        help="Show more output",
        nargs="?",
        const=True
        )
    parser.add_argument(
        "-pmc","--pm-command",
        help="Specifiy pm command (default: pm uninstall -k --user 0), all parameters after this will be consumed",
        nargs=argparse.REMAINDER,
        metavar="pm command",
        default=["pm", "uninstall", "-k", "--user", "0"] # -k = keep caches, --user 0 = force uninstall 
        )
    return(parser.parse_args())


def _run(cmd,aoe=True,cap_out=True):
    """ run shell command: cmd = List, aoe = Bool, cap_out = Bool """
    try:
        std = subprocess.run(cmd,check=aoe,capture_output=cap_out)
    except subprocess.CalledProcessError as e:
        return(e)

    #return(std.args,std.returncode)
    return(std)

def check_device(verbose):
    """ check for connected device """
    # TODO: allow device specification
    res = _run(["adb","devices"])
    
    if verbose: print(res);
    # general error
    if isinstance(res,subprocess.CalledProcessError):
        print("Error! %s (Exitcode %s)" %(res,res.returncode))
        return(False)
    
    # if adb devices command succed returncode is always 0, regardless if device is connected or not
    # so check for "device" string in stdout
    if "\tdevice\n" in res.stdout.decode(errors="ignore"): # device found
        # TEST
        #if args.verbose: print("device found");
        return(True)
    else:
        return(False)


def remove(pmc,pkg,verbose):
    """ remove pkg: pmc = list, pkg = string """
    cmd = ["adb","shell"] + pmc + [pkg]

    res = _run(cmd)
    if verbose: print("command was: %s\nresult is: %s" %(cmd,res));
    
    # general error
    if isinstance(res,subprocess.CalledProcessError):
        print("Error! %s (Exitcode: %s)" %(res,res.returncode))
        return((False,pkg))

    # adb shell always return 0 if device is connected and command was executed
    # so check for failure or success string
    # TODO:
    # find something more robust
    if "failure" in res.stdout.decode(errors="ignore").lower():
        # pkg failed
        return((False,pkg))
    elif "success" in res.stdout.decode(errors="ignore").lower():
        return((True,pkg))
    else:
        # should not happen
        return(None,pkg)

def _load_json(f,dump):
    """ load json file: f = String """
    try:
        with open(f) as jf:
            jobj = json.loads(jf.read())
            if dump:
                print( json.dumps(jobj, indent=4, separators=(",",":") ))
                # exit here
                exit(0)
            else:
                return(jobj)
    except FileNotFoundError:
        print("File %s not found!" %f)
        exit(3)

def _end():
    bo,t="*","Dont support Xiaomi and their crap. Better buy other devices"
    print("\n%s\n%s %s %s\n%s" %(bo*(len(t)+4),bo,t,bo,bo*(len(t)+4)))

def _show_configs(jobj):
    for i in jobj["config"]:
        try:
            comment = jobj["config"][i]["comment"]
        except KeyError:
            # comment missing in json file
            comment = None
        print("%s (%s)" %(i,comment) if comment is not None else "%s" %i)
    #print([(i,jobj["config"][i]["comment"]) for i in jobj["config"]])
    exit(2)

def main(args):
    if args.verbose: print("Passed args: %s" %(list((i,args.__dict__[i]) for i in args.__dict__)))
    #FMT = { "red":"\x1b[31m","green":"\x1b[32m","bold":"\x1b[1m","cyan":"\x1b[36m","default":"\x1b[0m" }

    pmc = args.pm_command # set default pm command
    
    if args.file is not None:
        # json specified
        jobj = _load_json(args.file,args.dump_json_file)
        if args.show_configs is not None:
            _show_configs(jobj)
        try:
            pkg_list = jobj["config"][args.config]["pkg_list"]
        except KeyError:
            print("config not found in %s\nAvailable:" %(args.file) )
            _show_configs(jobj)
        try:
            pmc = jobj["config"][args.config]["pm_command"]
        except KeyError:
            # use default pm command (already set above)
            if args.verbose: print("no pm_command key found in config '%s'. Using default %s" %(args.config,pmc));

    else:
        pkg_list = args.pkgs

    
    if args.skip_device_check is not True:
        if check_device(args.verbose) is False:
            print("No adb device found!")
            exit(1)

    if pkg_list == None:
        print("No pkgs specified, also no json file specified!")
        exit(1)
    suc,err = [],[]
    for i in pkg_list:
        res = remove(pmc,i,args.verbose)
        if res[0]:
            suc.append(res[1])
        else:
            err.append(res[1])
    
    print("success: %s" %suc)
    print("failure: %s" %err)
    
    _end()
    exit(0)

if __name__ == "__main__":
    main(create_parser())
